// This example illustrates how C++ classes can be used from Java using SWIG.
// The Java class gets mapped onto the C++ class and behaves as if it is a Java class.

import example;
import std.stdio;

int main() {
    // ----- Object creation -----

    writefln("Creating some objects:" );
    Circle c = new Circle(10);
    writefln("    Created circle ", c );
    Square s = new Square(10);
    writefln("    Created square ", s );

    // ----- Access a static member -----

    writefln("\nA total of ", Shape.nshapes, " shapes were created" );

    // ----- Member data access -----

    // Notice how we can do this using functions specific to
    // the 'Circle' class.
    c.x = 20;
    c.y = 30;

    // Now use the same functions in the base class
    Shape shape = s;
    shape.x = -1;
    shape.y = 5;

    writefln("\nHere is their current position:" );
    writefln("    Circle = (", c.x(), " ", c.y(), ")" );
    writefln("    Square = (", s.x(), " ", s.y(), ")" );

    // ----- Call some methods -----

    writefln("\nHere are some properties of the shapes:" );
    Shape[] shapes;
    shapes ~= c;
    shapes ~= s;
    foreach (Shape curShape; shapes) {
        with (curShape) {
            writefln("   ", toString() );
            writefln("        area      = ", area);
            writefln("        perimeter = ", perimeter);
        }
    }

    // Notice how the area() and perimeter() functions really
    // invoke the appropriate virtual method on each object.

    // ----- Delete everything -----

    writefln("\nGuess I'll clean up now" );

    // Note: this invokes the virtual destructor
    // You could leave this to the garbage collector
    delete c;
    delete s;

    writefln(Shape.nshapes, " shapes remain" );
    writefln("Goodbye" );
    return 0;
}
